@@ -7,20 +7,12 @@ import os
 import re
 import sys
 from collections import OrderedDict
+from difflib import SequenceMatcher
 from fnmatch import fnmatch
 from gettext import gettext as _
 from json import dumps
 from pathlib import Path
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Iterable,
-    Mapping,
-    MutableMapping,
-    cast,
-    no_type_check,
-)
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, MutableMapping, cast, no_type_check
 
 from packaging.specifiers import SpecifierSet
 from resolvelib.structs import DirectedGraph
@@ -83,19 +75,19 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
 
         # no help; start on same line and add a final newline
         if not action.help:
-            tup = self._current_indent, "", action_header
-            action_header = "%*s%s\n" % tup
+            tup = "", self._current_indent, action_header
+            action_header = "{0:>{1}}{2}\n".format(*tup)
 
         # short action name; start on the same line and pad two spaces
         elif len(action_header) <= action_width:
-            tup = self._current_indent, "", action_width, action_header  # type: ignore[assignment]
-            action_header = "%*s%-*s  " % tup  # type: ignore[str-format]
+            tup = "", self._current_indent, action_header, action_width  # type: ignore[assignment]
+            action_header = "{0:>{1}}{2:<{3}}  ".format(*tup)
             indent_first = 0
 
         # long action name; start on the next line
         else:
-            tup = self._current_indent, "", action_header
-            action_header = "%*s%s\n" % tup
+            tup = "", self._current_indent, action_header
+            action_header = "{0:>{1}}{2}\n".format(*tup)
             indent_first = help_position
 
         # Special format for empty action_header
@@ -113,9 +105,9 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
         if action.help:
             help_text = self._expand_help(action)
             help_lines = self._split_lines(help_text, help_width)
-            parts.append("%*s%s\n" % (indent_first, "", help_lines[0]))
+            parts.append("{:>{}}{}\n".format("", indent_first, help_lines[0]))
             for line in help_lines[1:]:
-                parts.append("%*s%s\n" % (help_position, "", line))
+                parts.append("{:>{}}{}\n".format("", help_position, line))
 
         # or add a newline if the description doesn't end with one
         elif not action_header.endswith("\n"):
@@ -156,6 +148,36 @@ class ArgumentParser(argparse.ArgumentParser):
         return args, argv
 
 
+def find_similar_text(origin_name: str, target_names: list[str], ratio: float = 0.5) -> list[str]:
+    results = []
+    for name in target_names:
+        ratio = SequenceMatcher(None, origin_name, name).ratio()
+        if ratio > 0.4:
+            results.append((name, ratio))
+    return [name for name, _ in sorted(results, key=lambda x: x[1], reverse=True)]
+
+
+def format_similar_command(root_command: str, commands: list[str], script_commands: list[str]) -> str:
+    similar_commands = find_similar_text(root_command, commands)
+    similar_script_commands = find_similar_text(root_command, script_commands)
+    commands_text = "\n".join([f"    - {cmd}" for cmd in similar_commands])
+    script_commands_text = "\n".join([f"    - {cmd}" for cmd in similar_script_commands])
+    message = f"""[red]Command not found: {root_command}[/]
+    """
+    if commands_text:
+        message += f"""
+[green]Did you mean one of these command?
+{commands_text}[/]
+"""
+
+    if script_commands_text:
+        message += f"""
+[yellow]Or one of these script command?
+{script_commands_text}[/]
+"""
+    return message
+
+
 class ErrorArgumentParser(ArgumentParser):
     """A subclass of argparse.ArgumentParser that raises
     parsing error rather than exiting.
@@ -164,10 +186,10 @@ class ErrorArgumentParser(ArgumentParser):
     """
 
     def _parse_known_args(
-        self, arg_strings: list[str], namespace: argparse.Namespace
+        self, arg_strings: list[str], namespace: argparse.Namespace, *args: Any, **kwargs: Any
     ) -> tuple[argparse.Namespace, list[str]]:
         try:
-            return super()._parse_known_args(arg_strings, namespace)
+            return super()._parse_known_args(arg_strings, namespace, *args, **kwargs)
         except argparse.ArgumentError as e:
             # We raise a dedicated error to avoid being caught by the caller
             raise PdmArgumentError(e) from e
@@ -526,7 +548,7 @@ def save_version_specifiers(
 
     :param requirements: the requirements to be updated
     :param resolved: the resolved mapping
-    :param save_strategy: compatible/wildcard/exact
+    :param save_strategy: compatible/safe_compatible/wildcard/exact
     """
 
     def candidate_version(candidates: list[Candidate]) -> Version | None:
@@ -544,11 +566,14 @@ def save_version_specifiers(
                 continue
             if save_strategy == "exact":
                 r.specifier = get_specifier(f"=={version}")
-            elif save_strategy == "compatible":
+            elif save_strategy in ["compatible", "safe_compatible"]:
                 if version.is_prerelease or version.is_devrelease:
                     r.specifier = get_specifier(f">={version},<{version.major + 1}")
                 else:
-                    r.specifier = get_specifier(f"~={version.major}.{version.minor}")
+                    if save_strategy == "compatible":
+                        r.specifier = get_specifier(f"~={version.major}.{version.minor}")
+                    else:
+                        r.specifier = get_specifier(f"~={version}")
             elif save_strategy == "minimum":
                 r.specifier = get_specifier(f">={version}")
 
