@@ -34,7 +34,6 @@ from pdm.utils import (
     filtered_sources,
     get_rev_from_url,
     normalize_name,
-    path_to_url,
     url_without_fragments,
 )
 
@@ -128,6 +127,7 @@ class Candidate:
         "link",
         "name",
         "req",
+        "requested",
         "summary",
         "version",
     )
@@ -153,6 +153,7 @@ class Candidate:
         self.link = link
         self.summary = ""
         self.hashes: list[FileHash] = []
+        self.requested = False
 
         self._requires_python: str | None = None
         self._prepared: PreparedCandidate | None = None
@@ -276,6 +277,7 @@ class Candidate:
                 result.update(revision=self.get_revision())
         elif not self.req.is_named:
             if self.req.is_file_or_url and self.req.is_local:
+                self.req._root = project_root
                 result.update(path=self.req.str_path)
             else:
                 result.update(url=self.req.url)
@@ -288,6 +290,8 @@ class Candidate:
     def prepare(self, environment: BaseEnvironment, reporter: CandidateReporter | None = None) -> PreparedCandidate:
         """Prepare the candidate for installation."""
         if self._prepared is None:
+            if isinstance(self.req, FileRequirement):
+                self.req.check_installable()
             self._prepared = PreparedCandidate(self, environment, reporter=reporter or CandidateReporter())
         else:
             self._prepared.environment = environment
@@ -351,7 +355,7 @@ class PreparedCandidate:
                 assert self._source_dir
                 return _filter_none(
                     {
-                        "url": path_to_url(self._source_dir.as_posix()),
+                        "url": self._source_dir.as_uri(),
                         "dir_info": {"editable": True},
                         "subdirectory": req.subdirectory,
                     }
