@@ -217,6 +217,12 @@ class Config(MutableMapping[str, str]):
             env_var="PDM_PYPI_JSON_API",
             coerce=ensure_boolean,
         ),
+        "scripts.show_header": ConfigItem(
+            "Display script name and help before running",
+            default=False,
+            env_var="PDM_SCRIPTS_SHOW_HEADER",
+            coerce=ensure_boolean,
+        ),
         "venv.location": ConfigItem(
             "Parent directory for virtualenvs",
             os.path.join(platformdirs.user_data_dir("pdm"), "venvs"),
@@ -320,9 +326,10 @@ class Config(MutableMapping[str, str]):
             repo = self.get_repository_config(parts[1], parts[0])
             if repo is None:
                 raise KeyError(f"No {parts[0]} named {parts[1]}")
-
-            value = getattr(repo, parts[2]) if len(parts) >= 3 else repo
-            return value
+            if len(parts) >= 3:
+                repo.populate_keyring_auth()
+                return getattr(repo, parts[2])
+            return repo
 
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
@@ -421,8 +428,6 @@ class Config(MutableMapping[str, str]):
 
     def get_repository_config(self, name_or_url: str, prefix: str) -> RepositoryConfig | None:
         """Get a repository or source by name or url."""
-        if not self.is_global and prefix == REPOSITORY:  # pragma: no cover
-            raise NoConfigError(prefix)
         repositories: dict[str, RepositoryConfig] = {}
         for k, v in self._data.items():
             if not k.startswith(f"{prefix}.") or k in self._config_map:
@@ -438,7 +443,7 @@ class Config(MutableMapping[str, str]):
         else:
             config = repositories.get(name_or_url)
 
-        if prefix == SOURCE:
+        if prefix == SOURCE or not self.is_global:
             return config
 
         if name_or_url in DEFAULT_REPOSITORIES:
