@@ -39,7 +39,6 @@ from pdm.utils import (
     is_conda_base_python,
     is_path_relative_to,
     normalize_name,
-    path_to_url,
 )
 
 if TYPE_CHECKING:
@@ -471,7 +470,6 @@ class Project:
             client_cert=self.config.get("pypi.client_cert"),
             client_key=self.config.get("pypi.client_key"),
         )
-        config.populate_auth()
         return config
 
     @property
@@ -505,7 +503,6 @@ class Project:
                 continue
             if expand_env:
                 source.url = DEFAULT_BACKEND(self.root).expand_line(expand_env_vars_in_auth(source.url))
-            source.populate_auth()
             sources.append(source)
         return sources
 
@@ -621,7 +618,7 @@ class Project:
 
         from pdm.models.candidates import Candidate
 
-        req = parse_requirement(path_to_url(self.root.as_posix()), editable)
+        req = parse_requirement(self.root.as_uri(), editable)
         assert self.name
         req.name = self.name
         can = Candidate(req, name=self.name, link=Link.from_path(self.root))
@@ -674,7 +671,8 @@ class Project:
         if group == "default":
             return metadata.get("dependencies", tomlkit.array()), lambda x: metadata.__setitem__("dependencies", x)
         dev_dependencies = self.pyproject._data.get("dependency-groups", {})
-        dev_dependencies.update(self.pyproject.settings.get("dev-dependencies", {}))
+        for group, items in self.pyproject.settings.get("dev-dependencies", {}).items():
+            dev_dependencies.setdefault(group, []).extend(items)
         deps_setter = [
             (
                 metadata.get("optional-dependencies", {}),
@@ -792,13 +790,26 @@ class Project:
         python_spec: str | None = None,
         search_venv: bool | None = None,
         filter_func: Callable[[PythonInfo], bool] | None = None,
+        respect_version_file: bool = True,
     ) -> Iterable[PythonInfo]:
         """Iterate over all interpreters that matches the given specifier.
         And optionally install the interpreter if not found.
         """
+        from packaging.version import InvalidVersion
+
         from pdm.cli.commands.python import InstallCommand
 
         found = False
+        if (
+            respect_version_file
+            and not python_spec
+            and (os.getenv("PDM_PYTHON_VERSION") or self.root.joinpath(".python-version").exists())
+        ):
+            requested = os.getenv("PDM_PYTHON_VERSION") or self.root.joinpath(".python-version").read_text().strip()
+            if requested not in self.python_requires:
+                self.core.ui.warn(".python-version is found but the version is not in requires-python, ignored.")
+            else:
+                python_spec = requested
         for interpreter in self.find_interpreters(python_spec, search_venv):
             if filter_func is None or filter_func(interpreter):
                 found = True
@@ -812,7 +823,12 @@ class Project:
             if best_match is None:
                 return
             python_spec = str(best_match)
-
+        else:
+            try:
+                if python_spec not in self.python_requires:
+                    return
+            except InvalidVersion:
+                return
         try:
             # otherwise if no interpreter is found, try to install it
             installed = InstallCommand.install_python(self, python_spec)
